@@ -25,8 +25,10 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.LongAdder;
 
+import com.github.luben.zstd.ZstdException;
 import com.google.common.util.concurrent.Uninterruptibles;
 import io.netty.buffer.ByteBuf;
+import net.jpountz.lz4.LZ4Exception;
 import org.roaringbitmap.RoaringBitmap;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -644,8 +646,20 @@ public abstract class CelebornInputStream extends InputStream {
         }
 
         return hasData;
-      } catch (IOException e) {
-        IOException ioe = e;
+      } catch (LZ4Exception | ZstdException | IOException e) {
+        logger.error(
+            "Failed to fill buffer from chunk. AppShuffleId {}, shuffleId {}, partitionId {}, location {}",
+            appShuffleId,
+            shuffleId,
+            partitionId,
+            currentReader.getLocation(),
+            e);
+        IOException ioe;
+        if (e instanceof IOException) {
+          ioe = (IOException) e;
+        } else {
+          ioe = new IOException(e);
+        }
         if (exceptionMaker != null) {
           if (shuffleClient.reportShuffleFetchFailure(appShuffleId, shuffleId)) {
             /*
@@ -660,6 +674,15 @@ public abstract class CelebornInputStream extends InputStream {
           }
         }
         throw ioe;
+      } catch (Exception e) {
+        logger.error(
+            "Failed to fill buffer from chunk. AppShuffleId {}, shuffleId {}, partitionId {}, location {}",
+            appShuffleId,
+            shuffleId,
+            partitionId,
+            currentReader.getLocation(),
+            e);
+        throw e;
       }
     }
 
