@@ -563,7 +563,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         replyRegisterShuffle(RegisterShuffleResponse(StatusCode.SLOT_NOT_AVAILABLE, Array.empty))
         return
       case StatusCode.SUCCESS =>
-        logInfo(s"OfferSlots for $shuffleId Success!Slots Info: ${res.workerResource}")
+        logDebug(s"OfferSlots for $shuffleId Success!Slots Info: ${res.workerResource}")
       case StatusCode.WORKER_EXCLUDED =>
         logInfo(s"OfferSlots for $shuffleId failed due to all workers be excluded!")
         replyRegisterShuffle(RegisterShuffleResponse(StatusCode.WORKER_EXCLUDED, Array.empty))
@@ -743,8 +743,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       return
     }
 
-    def isAllMaptaskEnd(shuffleId: Int): Boolean = {
-      !commitManager.getMapperAttempts(shuffleId).exists(_ < 0)
+    def areAllMapTasksEnd(shuffleId: Int): Boolean = {
+      ClientUtils.areAllMapperAttemptsFinished(commitManager.getMapperAttempts(shuffleId))
     }
 
     shuffleIds.synchronized {
@@ -781,7 +781,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
                 s"unexpected! unknown appShuffleId $appShuffleId when checking shuffle deterministic level"))
         }
       } else {
-        shuffleIds.values.map(v => v._1).toSeq.reverse.find(isAllMaptaskEnd) match {
+        shuffleIds.values.map(v => v._1).toSeq.reverse.find(areAllMapTasksEnd) match {
           case Some(shuffleId) =>
             val pbGetShuffleIdResponse = {
               logDebug(
