@@ -516,7 +516,7 @@ final private[worker] class StorageManager(conf: CelebornConf, workerSource: Abs
         keyedWriters._1 -> keyedWriters._2.values().asScala.map(_.getFileLength).sum
       }
     }.toList.map { case (shuffleKey, usage) =>
-      shuffleKey.split("-")(0) -> usage
+      Utils.splitShuffleKey(shuffleKey)._1 -> usage
     }.groupBy(_._1).map { case (key, values) =>
       key -> values.map(_._2).sum
     }.toSeq.sortBy(_._2).reverse.take(conf.metricsAppTopDiskUsageCount * 2).toMap.asJava
